# Desactiva el bytecode de Python
import sys
sys.dont_write_bytecode = True

# Librerías
import glob
import shutil
import argparse
import requests
from os import cpu_count
from datetime import date
import zipfile_deflate64 as zipfile

# Ajustes
from Ajustes import *

# Se configuran los logs
Log.configurar_logs(INSTALACION_SCRIPT_LOG)

# Interfaz de línea de comandos
parser = argparse.ArgumentParser()
parser.add_argument('-b', '--base', help='Archivo base', type=str)
parser.add_argument('-d', '--diff', help='Archivo diff', type=str)
parser.add_argument('-e', '--especial', help='Dia especial', action='store_true')
parser.add_argument('-n', '--no-descargar', help='No descargar archivo de datos del día actual', action='store_true')
argumentos = parser.parse_args()

# Variables de la interfaz de línea de comandos
DIA_ESPECIAL: bool = argumentos.especial
DESCARGAR_ACTUAL: bool = not argumentos.no_descargar

# Se genera la ruta del archivo de datos del día actual
NOMBRE_ARCHIVO = 'datos_abiertos_' + date.today().strftime('%Y%m%d') + '.zip'
ARCHIVO_DIA_ACTUAL = INSTALACION_DATOS_COVID / NOMBRE_ARCHIVO

# Solo se descarga el archivo de datos del día actual si no existe
if DESCARGAR_ACTUAL and not ARCHIVO_DIA_ACTUAL.exists():

	# Se descarga el archivo de datos del día actual
	exito = Log.subproceso(['curl', ajuste('url_datos_dge'),
		'-o', str(ARCHIVO_DIA_ACTUAL)],
		True, redirigir_stderr= True)

	# Se comprueba que la descarga haya sido exitosa
	if not exito:

		# Se borra el archivo de datos del día actual ya que probablemente esté corrupto
		if ARCHIVO_DIA_ACTUAL.exists():
			ARCHIVO_DIA_ACTUAL.unlink()

		Log.error(f'No se pudo descargar el archivo de datos del día actual')
	else:
		print('Archivo de datos del día actual descargado')

# Se obtiene una lista de todos los archivos de datos
ARCHIVOS_COVID = sorted(glob.glob(f'{INSTALACION_DATOS_COVID}/datos_abiertos_*.zip'))

# Se obtiene el archivo base y el archivo de diferencias
ARCHIVO_BASE = ARCHIVOS_COVID[-2] if argumentos.base is None else argumentos.base
ARCHIVO_DIFF = ARCHIVOS_COVID[-1] if argumentos.diff is None else argumentos.diff

print(f'Archivo base: {ARCHIVO_BASE}')
print(f'Archivo diff: {ARCHIVO_DIFF}')

# Archivos de diferencias
ARCHIVO_INSERT = CARPETA_TEMPORAL / 'insert.csv'
ARCHIVO_DELETE = CARPETA_TEMPORAL / 'delete.csv'
ARCHIVO_UPDATE = CARPETA_TEMPORAL / 'update.csv'

# Se descomprime el archivo base
with zipfile.ZipFile(ARCHIVO_BASE, 'r') as archivo_zip:
	zipinfo = archivo_zip.infolist()[0]
	zipinfo.filename += '_base'
	archivo_zip.extract(zipinfo, CARPETA_TEMPORAL)
	ARCHIVO_BASE_TEMPORAL = CARPETA_TEMPORAL / zipinfo.filename

# Se descomprime el archivo diff
with zipfile.ZipFile(ARCHIVO_DIFF, 'r') as archivo_zip:
	zipinfo = archivo_zip.infolist()[0]
	zipinfo.filename += '_diff'
	archivo_zip.extract(zipinfo, CARPETA_TEMPORAL)
	ARCHIVO_DIFF_TEMPORAL = CARPETA_TEMPORAL / zipinfo.filename

# Se hace la diferencia de ambos archivos
exito = Log.subproceso(
[
	INSTALACION_CSVDIFF, '-cfu',

	# Se usan todos los núcleos disponibles
	'-t', str(cpu_count()),

	# La columna con índice 1 (ID_REGISTRO) es la llave primaria
	'-p', '1',

	# La columna con índice 0 (FECHA_ACTUALIZACION) se ignora
	'-i', '0',

	# Ubicación de los archivos de diferencias
	'-I', str(ARCHIVO_INSERT),
	'-D', str(ARCHIVO_DELETE),
	'-U', str(ARCHIVO_UPDATE),

	# Archivo base y de diferencias
	str(ARCHIVO_BASE_TEMPORAL),
	str(ARCHIVO_DIFF_TEMPORAL)
],
True)

# Se borran los archivos temporales
ARCHIVO_BASE_TEMPORAL.unlink()
ARCHIVO_DIFF_TEMPORAL.unlink()

# Se comprueba que la diferencia haya sido exitosa
if not exito:
	Log.error('No se pudo obtener la diferencia de los archivos')

# Se autodetecta el día especial
if ajuste('dia_especial.autodetectar') and not DIA_ESPECIAL:
	with open(ARCHIVO_DELETE) as archivo:
		cantidad_eliminados = sum(1 for _ in archivo)

	# Si se sobrepasa el límite de eliminados, se marca como día especial
	if cantidad_eliminados > ajuste('dia_especial.eliminados'):
		print('Se detecto un día especial')
		DIA_ESPECIAL = True

# Se asegura que la carpeta de diferencias este vacía
if any(INSTALACION_DIFERENCIAS.iterdir()):
	print('Se detectaron archivos de diferencias previos')

	# Se manda una solicitud POST a Kafka Connect para reiniciar el conector
	respuesta = requests.post(ruta_kafka('kafka_connect') +
		'/connectors/' + ajuste('kafka.conector') + '/restart')

	# Se comprueba la respuesta de la solicitud
	if not respuesta.ok:
		Log.error('No se pudo reiniciar el conector de FilePulse')

	# Se termina el script con un error
	Log.error('La carpeta de diferencias no esta vacia')
else:

	# Se mueven los archivos de diferencias
	shutil.move(str(ARCHIVO_INSERT), str(INSTALACION_DIFERENCIAS))
	shutil.move(str(ARCHIVO_UPDATE), str(INSTALACION_DIFERENCIAS))

	# El archivo de registros eliminados solo se mueve si no es día especial
	if not DIA_ESPECIAL:
		shutil.move(str(ARCHIVO_DELETE), str(INSTALACION_DIFERENCIAS))
	else:
		ARCHIVO_DELETE.unlink()